@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2011 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -15,13 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections.abc
 import copy
 import math
+from unittest import mock
 
 import ddt
-import mock
 from oslotest import base as test_base
-import six
 import testscenarios
 
 from oslo_utils import strutils
@@ -32,12 +30,6 @@ load_tests = testscenarios.load_tests_apply_scenarios
 
 class StrUtilsTest(test_base.BaseTestCase):
 
-    @mock.patch("six.text_type")
-    def test_bool_bool_from_string_no_text(self, mock_text):
-        self.assertTrue(strutils.bool_from_string(True))
-        self.assertFalse(strutils.bool_from_string(False))
-        self.assertEqual(0, mock_text.call_count)
-
     def test_bool_bool_from_string(self):
         self.assertTrue(strutils.bool_from_string(True))
         self.assertFalse(strutils.bool_from_string(False))
@@ -84,15 +76,15 @@ class StrUtilsTest(test_base.BaseTestCase):
         self._test_bool_from_string(lambda s: s)
 
     def test_unicode_bool_from_string(self):
-        self._test_bool_from_string(six.text_type)
-        self.assertFalse(strutils.bool_from_string(u'使用', strict=False))
+        self._test_bool_from_string(str)
+        self.assertFalse(strutils.bool_from_string('使用', strict=False))
 
         exc = self.assertRaises(ValueError, strutils.bool_from_string,
-                                u'使用', strict=True)
-        expected_msg = (u"Unrecognized value '使用', acceptable values are:"
-                        u" '0', '1', 'f', 'false', 'n', 'no', 'off', 'on',"
-                        u" 't', 'true', 'y', 'yes'")
-        self.assertEqual(expected_msg, six.text_type(exc))
+                                '使用', strict=True)
+        expected_msg = ("Unrecognized value '使用', acceptable values are:"
+                        " '0', '1', 'f', 'false', 'n', 'no', 'off', 'on',"
+                        " 't', 'true', 'y', 'yes'")
+        self.assertEqual(expected_msg, str(exc))
 
     def test_other_bool_from_string(self):
         self.assertFalse(strutils.bool_from_string(None))
@@ -169,16 +161,17 @@ class StrUtilsTest(test_base.BaseTestCase):
     def test_slugify(self):
         to_slug = strutils.to_slug
         self.assertRaises(TypeError, to_slug, True)
-        self.assertEqual(six.u("hello"), to_slug("hello"))
-        self.assertEqual(six.u("two-words"), to_slug("Two Words"))
-        self.assertEqual(six.u("ma-any-spa-ce-es"),
+        self.assertEqual("hello", to_slug("hello"))
+        self.assertEqual("two-words", to_slug("Two Words"))
+        self.assertEqual("ma-any-spa-ce-es",
                          to_slug("Ma-any\t spa--ce- es"))
-        self.assertEqual(six.u("excamation"), to_slug("exc!amation!"))
-        self.assertEqual(six.u("ampserand"), to_slug("&ampser$and"))
-        self.assertEqual(six.u("ju5tnum8er"), to_slug("ju5tnum8er"))
-        self.assertEqual(six.u("strip-"), to_slug(" strip - "))
-        self.assertEqual(six.u("perche"), to_slug(six.b("perch\xc3\xa9")))
-        self.assertEqual(six.u("strange"),
+        self.assertEqual("excamation", to_slug("exc!amation!"))
+        self.assertEqual("ampserand", to_slug("&ampser$and"))
+        self.assertEqual("ju5tnum8er", to_slug("ju5tnum8er"))
+        self.assertEqual("strip-", to_slug(" strip - "))
+        self.assertEqual("perche",
+                         to_slug("perch\xc3\xa9".encode("latin-1")))
+        self.assertEqual("strange",
                          to_slug("\x80strange", errors="ignore"))
 
 
@@ -214,10 +207,22 @@ class StringToBytesTest(test_base.BaseTestCase):
         ('M', dict(unit_prefix='M')),
         ('G', dict(unit_prefix='G')),
         ('T', dict(unit_prefix='T')),
+        ('P', dict(unit_prefix='P')),
+        ('E', dict(unit_prefix='E')),
+        ('Z', dict(unit_prefix='Z')),
+        ('Y', dict(unit_prefix='Y')),
+        ('R', dict(unit_prefix='R')),
+        ('Q', dict(unit_prefix='Q')),
         ('Ki', dict(unit_prefix='Ki')),
         ('Mi', dict(unit_prefix='Mi')),
         ('Gi', dict(unit_prefix='Gi')),
         ('Ti', dict(unit_prefix='Ti')),
+        ('Pi', dict(unit_prefix='Pi')),
+        ('Ei', dict(unit_prefix='Ei')),
+        ('Zi', dict(unit_prefix='Zi')),
+        ('Yi', dict(unit_prefix='Yi')),
+        ('Ri', dict(unit_prefix='Ri')),
+        ('Qi', dict(unit_prefix='Qi')),
         ('invalid_unit_prefix', dict(unit_prefix='B', assert_error=True)),
     ]
 
@@ -290,11 +295,29 @@ class StringToBytesTest(test_base.BaseTestCase):
         else:
             self.assertAlmostEqual(actual, expected)
 
+
 StringToBytesTest.generate_scenarios()
 
 
 class MaskPasswordTestCase(test_base.BaseTestCase):
 
+    def test_namespace_objects(self):
+        payload = """
+        Namespace(passcode='', username='', password='my"password',
+        profile='', verify=None, token='')
+        """
+        expected = """
+        Namespace(passcode='', username='', password='***',
+        profile='', verify=None, token='***')
+        """
+        self.assertEqual(expected, strutils.mask_password(payload))
+
+    def test_sanitize_keys(self):
+
+        lowered = [k.lower() for k in strutils._SANITIZE_KEYS]
+        message = "The _SANITIZE_KEYS must all be lowercase."
+        self.assertEqual(strutils._SANITIZE_KEYS, lowered, message)
+
     def test_json(self):
         # Test 'adminPass' w/o spaces
         payload = """{'adminPass':'TL0EfN33'}"""
@@ -352,6 +375,43 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         payload = """{ 'token' : 'token' }"""
         expected = """{ 'token' : '***' }"""
         self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'fernetkey'
+        payload = """{ 'fernetkey' : 'token' }"""
+        expected = """{ 'fernetkey' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'FernetKey'
+        payload = """{ 'FernetKey' : 'token' }"""
+        expected = """{ 'FernetKey' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'sslkey'
+        payload = """{ 'sslkey' : 'token' }"""
+        expected = """{ 'sslkey' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'SslKey'
+        payload = """{ 'SslKey' : 'token' }"""
+        expected = """{ 'SslKey' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'passphrase'
+        payload = """{ 'passphrase' : 'token' }"""
+        expected = """{ 'passphrase' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'PassPhrase'
+        payload = """{ 'PassPhrase' : 'token' }"""
+        expected = """{ 'PassPhrase' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Some real-life cases
+        # Test 'KeystoneFernetKey1'
+        payload = """{ 'KeystoneFernetKey1' : 'token' }"""
+        expected = """{ 'KeystoneFernetKey1' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'OctaviaCaKeyPassword'
+        payload = """{ 'OctaviaCaKeyPassword' : 'token' }"""
+        expected = """{ 'OctaviaCaKeyPassword' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'OctaviaCaKeyPassphrase'
+        payload = """{ 'OctaviaCaKeyPassphrase' : 'token' }"""
+        expected = """{ 'OctaviaCaKeyPassphrase' : '***' }"""
+        self.assertEqual(expected, strutils.mask_password(payload))
 
     def test_xml(self):
         # Test 'adminPass' w/o spaces
@@ -394,6 +454,10 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
                      </password>"""
         expected = """<password>***</password>"""
         self.assertEqual(expected, strutils.mask_password(payload))
+        # Test 'Password1' - case-insensitive + number
+        payload = """<Password1>TL0EfN33</Password1>"""
+        expected = """<Password1>***</Password1>"""
+        self.assertEqual(expected, strutils.mask_password(payload))
 
     def test_xml_attribute(self):
         # Test 'adminPass' w/o spaces
@@ -566,13 +630,22 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         expected = 'test = "param1" : "value"'
         self.assertEqual(expected, strutils.mask_password(payload))
 
+        payload = 'test = "original_password" : "aaaaa"aaaa"'
+        expected = 'test = "original_password" : "***"'
+        self.assertEqual(expected, strutils.mask_password(payload))
+
         payload = """{'adminPass':'TL0EfN33'}"""
-        payload = six.text_type(payload)
+        payload = str(payload)
+        expected = """{'adminPass':'***'}"""
+        self.assertEqual(expected, strutils.mask_password(payload))
+
+        payload = """{'adminPass':'TL0E'fN33'}"""
+        payload = str(payload)
         expected = """{'adminPass':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
 
         payload = """{'token':'mytoken'}"""
-        payload = six.text_type(payload)
+        payload = str(payload)
         expected = """{'token':'***'}"""
         self.assertEqual(expected, strutils.mask_password(payload))
 
@@ -613,6 +686,31 @@ class MaskPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected, strutils.mask_password(payload))
 
 
+class TestMapping(collections.abc.Mapping):
+    """Test class for non-dict mappings"""
+    def __init__(self):
+        super(TestMapping, self).__init__()
+        self.data = {'password': 'shhh',
+                     'foo': 'bar',
+                     }
+
+    def __getitem__(self, key):
+        return self.data[key]
+
+    def __iter__(self):
+        return self.data.__iter__()
+
+    def __len__(self):
+        return len(self.data)
+
+
+class NestedMapping(TestMapping):
+    """Test class that contains an instance of TestMapping"""
+    def __init__(self):
+        super(NestedMapping, self).__init__()
+        self.data = {'nested': TestMapping()}
+
+
 class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
 
     def test_dictionary(self):
@@ -621,6 +719,11 @@ class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected,
                          strutils.mask_dict_password(payload))
 
+        payload = {'password': 'TL0Ef"N33'}
+        expected = {'password': '***'}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
         payload = {'user': 'admin', 'password': 'TL0EfN33'}
         expected = {'user': 'admin', 'password': '***'}
         self.assertEqual(expected,
@@ -647,6 +750,16 @@ class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected,
                          strutils.mask_dict_password(payload))
 
+        payload = {'passwords': {'KeystoneFernetKey1': 'c5FijjS'}}
+        expected = {'passwords': {'KeystoneFernetKey1': '***'}}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
+        payload = {'passwords': {'keystonecredential0': 'c5FijjS'}}
+        expected = {'passwords': {'keystonecredential0': '***'}}
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
     def test_do_no_harm(self):
         payload = {}
         expected = {}
@@ -660,6 +773,13 @@ class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
         self.assertEqual(expected,
                          strutils.mask_dict_password(payload))
 
+    def test_do_an_int(self):
+        payload = {}
+        payload[1] = 2
+        expected = payload.copy()
+        self.assertEqual(expected,
+                         strutils.mask_dict_password(payload))
+
     def test_mask_values(self):
         payload = {'somekey': 'test = cmd --password my\xe9\x80\x80pass'}
         expected = {'somekey': 'test = cmd --password ***'}
@@ -689,6 +809,21 @@ class MaskDictionaryPasswordTestCase(test_base.BaseTestCase):
         strutils.mask_dict_password(payload)
         self.assertEqual(pristine, payload)
 
+    def test_non_dict(self):
+        expected = {'password': '***',
+                    'foo': 'bar',
+                    }
+        payload = TestMapping()
+        self.assertEqual(expected, strutils.mask_dict_password(payload))
+
+    def test_nested_non_dict(self):
+        expected = {'nested': {'password': '***',
+                               'foo': 'bar',
+                               }
+                    }
+        payload = NestedMapping()
+        self.assertEqual(expected, strutils.mask_dict_password(payload))
+
 
 class IsIntLikeTestCase(test_base.BaseTestCase):
     def test_is_int_like_true(self):
@@ -841,7 +976,7 @@ class ValidateIntegerTestCase(test_base.BaseTestCase):
                "min_value": 300, "max_value": 300},
               {"value": 55, "name": "doing 55 in a 54",
                "max_value": 54},
-              {"value": six.unichr(129), "name": "UnicodeError",
+              {"value": chr(129), "name": "UnicodeError",
                "max_value": 1000})
     def test_invalid_inputs(self, value, name, **kwargs):
         self.assertRaises(ValueError, strutils.validate_integer,
